﻿using Mysqlx.Crud;
using StudentManager.Common;
using StudentManager.Data;
using StudentManager.Extensions;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace StudentManager.Model
{
	class MenuBar : BindableBase
	{
		private string icon;
		/// <summary>
		/// Icon
		/// </summary>
		public string Icon
		{
			get { return icon; }
			set { icon = value; }
		}

		private string title;

		public string Title
		{
			get { return title; }
			set { title = value; }
		}


		private string nameSpace;

		public string NameSpace
		{
			get { return nameSpace; }
			set { nameSpace = value; }
		}


	}

	class Students : BindableBase
	{

		private ObservableCollection<StudentData> studentDatas = new ObservableCollection<StudentData>();
		public ReadOnlyObservableCollection<StudentData> StudentDatas { get; private set; }

		private ObservableCollection<QuestionData> questionDatas = new ObservableCollection<QuestionData>();
		public ReadOnlyObservableCollection<QuestionData> QuestionDatas { get; private set; }

		private ObservableCollection<QuestionData> studentQuestionDatas = new ObservableCollection<QuestionData>();
		public ReadOnlyObservableCollection<QuestionData> StudentQuestionDatas { get; private set; }

		private ObservableCollection<CursonQuestionsData> cQDatas = new ObservableCollection<CursonQuestionsData>();
		public ReadOnlyObservableCollection<CursonQuestionsData> CQDatas { get; private set; }


		private ObservableCollection<QuestionData> studentCQDatas = new ObservableCollection<QuestionData>();
		public ReadOnlyObservableCollection<QuestionData> StudentCQDatas { get; private set; }

		private ObservableCollection<MenuBar> menuBars = new ObservableCollection<MenuBar>();
		public ReadOnlyObservableCollection<MenuBar> MenuBars { get; private set; }



		private int totalQuestions;
		public int TotalQuestions
		{
			get { return totalQuestions; }
			set { totalQuestions = value; }
		}


		private StudentData selectedStudent;
		public StudentData SelectedStudent
		{
			get { return selectedStudent; }
			set { selectedStudent = value; RaisePropertyChanged(); }
		}


		private CursonQuestionsData selectedCQData;
		public CursonQuestionsData SelectedCQData
		{
			get { return selectedCQData; }
			set { selectedCQData = value; RaisePropertyChanged(); }
		}


		private readonly IRegionManager regionManager;

		public DelegateCommand<string> RegionTo { get; set; }
		public DelegateCommand SelectedCommand { get; set; }
		public DelegateCommand<CursonQuestionsData> CQSelectedCommand { get; set; }
		public DelegateCommand RegionToDetailCommand { get; set; }

		Students(IRegionManager regionManager)
		{
			CreateMenuBar();

			RegionTo = new DelegateCommand<string>((x) => {
				regionManager.Regions[PrismManager.MainRegionName].RequestNavigate(x.ToString()); 
				if(x == "DetailCheckView")
				{
					RegionToDCV();
				}
				RegionToStudents(); 
			});

			SelectedCommand = new DelegateCommand(() =>
			{
				cQDatas.Clear();
				SQLHelper.Query<CursonQuestionsData>(Tables.CQTable, SelectedStudent.UID).ForEach(x => cQDatas.Add(x));
			});

			CQSelectedCommand = new DelegateCommand<CursonQuestionsData>((e) =>
			{
				selectedCQData = e;
			});

			StudentDatas = new ReadOnlyObservableCollection<StudentData>(studentDatas);
			QuestionDatas = new ReadOnlyObservableCollection<QuestionData>(questionDatas);
			StudentQuestionDatas = new ReadOnlyObservableCollection<QuestionData>(studentQuestionDatas);
			MenuBars = new ReadOnlyObservableCollection<MenuBar>(menuBars);
			CQDatas = new ReadOnlyObservableCollection<CursonQuestionsData>(cQDatas);
			StudentCQDatas = new ReadOnlyObservableCollection<QuestionData>(studentCQDatas);




			this.regionManager = regionManager;

			QueryAll();



			StudentLib.Test();
			StudentLib.Save();

			QuestionLib.Test();
			QuestionLib.Save();
		}

		private void RegionToDCV()
		{
			Debug.Assert(SelectedCQData != null);
			GetQuestions(SelectedCQData.ProblemIDS);			
		}

		private void RegionToStudents()
		{
			
		}

		void CreateMenuBar()
		{
			menuBars.Add(new MenuBar() { Icon = "ForumPlusOutline", Title = "学生", NameSpace = "StudentsView" });
			menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "详情", NameSpace = "DetailView" });
			menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "题库", NameSpace = "QuestionsView" });
			menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "审批", NameSpace = "CheckView" });
			//menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "出题", NameSpace = "QuestionsView" });
			menuBars.Add(new MenuBar() { Icon = "CodeGreaterThanOrEqual", Title = "出题", NameSpace = "DetailCheckView" });
		}


		private void QueryAll()
		{
			studentDatas.Clear();
			questionDatas.Clear();
			//cQDatas.Clear();
			//SQLHelper.Query<CursonQuestionsData>(Tables.CQTable,1).ForEach(x => cQDatas.Add(x));
			SQLHelper.Query<StudentData>(Tables.UserTable).ForEach(x => studentDatas.Add(x));
			SQLHelper.Query<QuestionData>(Tables.QuesTable).ForEach(x => questionDatas.Add(x));


			
			ComputeData();
			//SQLHelper.UnionQuery<QuestionData>().ForEach(x => questionDatas.Add(x)); 
		}

		private void ComputeData()
		{
			TotalQuestions = questionDatas.Count();

            foreach (var item in StudentDatas)
            {
				item.UnCorrectCount = item.TotalsQuestions - item.CorrectionCount;
				item.ErrorRate = item.TotalsQuestions / TotalQuestions;
				item.CorrectRate = item.CorrectionCount / item.TotalsQuestions;
            }
        }

		private void GetQuestions(int[] ids)
		{
			if (SelectedCQData == null) return;
			studentCQDatas.Clear();
			foreach (var id in ids)
			{
				var q = QuestionDatas.FirstOrDefault((s) => s.Id == id);
				studentCQDatas.Add(q);
			}
		}
	}
}
